# -*- coding: utf-8 -*-

from message import Message
from sqlalchemy import create_engine
from sqlalchemy.orm import mapper
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy import Table, Column, Integer, String, Text, TIMESTAMP, MetaData

from logging import getLogger
log = getLogger('su.artix.queue')

class Dao(object):
    """
    Класс для работы с базой данных
    """
    def __init__(self):
        # создадим регистр сессий, в котором каждый обработчик будет создавать свою сессию
        self.Session = self.createScopedSession()

    def hasMessage(self, session, messageType):
        """
        Проверка наличия в очереди сообщений указанного типа
        """
        if session.query(Message).filter_by(type=messageType).first():
            return True
        return False

    def loadMessages(self, session, messagesType=None):
        """
        Загрузка всех сообщений указанного типа
        """
        if messagesType:
            return session.query(Message).filter_by(type=messagesType).all()
        return session.query(Message).all()

    def loadMessage(self, session, messageType=None, lastFailId=None, skipErrorMessage = True):
        """
        Загрузка перевого сообщения указанного типа
        """
        if messageType:
            if lastFailId and skipErrorMessage:
                log.debug(u"Загрузка сообщения типа %s с messageid > %s" % (messageType, lastFailId))
                return session.query(Message).filter_by(type=messageType).filter(Message.messageid > lastFailId).first()
            return session.query(Message).filter_by(type=messageType).first()
        return session.query(Message).first()

    def deleteMessage(self, session, messageid):
        session.query(Message).filter(Message.messageid == messageid).delete(False)

    def createScopedSession(self):
        messagestable = Table("message", MetaData(),
                              Column('messageid', Integer, primary_key=True),
                              Column('type', String(30)),
                              Column('body', Text),
                              Column('creationtime', TIMESTAMP))

        mapper(Message, messagestable)
        engine = create_engine('mysql://root@localhost/queues?charset=utf8', echo=False)
        Session = scoped_session(sessionmaker(engine, autoflush=True, autocommit=True))

        return Session

    def createSession(self):
        return self.Session()
