# -*- coding: utf8 -*-

from logging import getLogger
from base import BaseHandler
import httplib2
import json

log = getLogger('su.artix.queue')

class AbmLoyaltyHandler(BaseHandler):
    """
    Обработчик сообщений AbmLoyalty
    """
    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(AbmLoyaltyHandler, self).__init__(dao, stopThreadsFlag, lastFailsLock, lastFails)
        self.messageType = "abmloyalty"
        self.name = "AbmLoyaltyHandler"

    def createConnection(self, timeout):
        return httplib2.Http(timeout = timeout, disable_ssl_certificate_validation = True)

    def sendMessage(self, message):
        log.info(u'Обработка сообщения %s, %s' % (message.messageid, message.body))
        try:
            result, content = self.send(message)
            if result['status'] == '201':
                self.deleteSentMessage(message)
                log.info(u'Сообщение %s успешно отправлено' % message.messageid)
                return True
            else:
                log.error(u'Ошибка REST-запроса, код ошибки: %s' % result['status'])
        except Exception as e:
            log.error(u'Ошибка: %s' % e)
        return False

    def send(self, message):
        content = json.loads(message.body)
        method = str(content['method'])
        headers = content['headers']
        body = str(content['body'])
        timeout = int(content['timeout'])
        url = str(content['url'])

        log.info(u'Отправка на адрес %s сообщения: %s' % (url, body))
        connection = self.createConnection(timeout)
        result, answer = connection.request(url, method, headers = headers, body = body)
        log.info(u'Ответ от сервера: %s , %s' % (result, answer.decode('utf-8')))
        return result, answer
