# -*- coding: utf8 -*-
from threading import Thread
from logging import getLogger

log = getLogger('su.artix.queue')

class BaseHandler(Thread):
    """
    Базовый класс обработчика сообщений
    """
    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(BaseHandler, self).__init__()
        self.dao = dao
        self.stopThreadsFlag = stopThreadsFlag
        self.lastFailsLock = lastFailsLock
        self.lastFails = lastFails
        self.skipErrorMessage = True

    def run(self):
        log.info(u"Обработчик %s запущен" % self.name)
        self.handleMessages()

    def handleMessages(self):
        """
        Функция получения сообщений, соответсвующих типу обработчика
        Будет выполняться, пока сообщения не закончатся, пока не появится ошибка отправки,
        или пока очередь не выставит флаг завершения работы.
        В случае ошибки, поток дождется освобождения общего ресурса для записи messageid неотправленного сообщения
        """
        while not self.stopThreadsFlag.isSet():
            message = self.loadMessageFromDb()
            if message:
                if not self.sendMessage(message):
                    with self.lastFailsLock:
                        self.lastFails[self.messageType] = message.messageid
                    break
            else:
                break

        self.dao.Session.remove()
        log.info(u"Обработчик %s завершил обработку сообщений типа %s" % (self.name, self.messageType))

    def loadMessageFromDb(self):
        lastFailId = None
        message = None

        with self.lastFailsLock:
            if self.messageType in self.lastFails:
                lastFailId = self.lastFails[self.messageType]
                log.debug(u"Последнее неудачное сообщение %s: %s" % (self.messageType, lastFailId))

        if lastFailId:
            message = self.dao.loadMessage(self.dao.Session, self.messageType, self.lastFails[self.messageType], self.skipErrorMessage)
            if not message:
                with self.lastFailsLock:
                    del self.lastFails[self.messageType]
        else:
            message = self.dao.loadMessage(self.dao.Session, self.messageType)

        return message

    def sendMessage(self, message):
        """
        Функция отправки сообщений
        """
        pass

    def deleteSentMessage(self, message):
        """
        Удаление обработанного сообщения из базы данных
        """
        log.debug(u"Удаление отправленного сообщения %s типа %s" % (message.messageid, message.type))
        self.dao.deleteMessage(self.dao.Session, message.messageid)
