# -*- coding: utf8 -*-

from logging import getLogger
from base import BaseHandler
import json
import os
import httplib2
import xml.etree.ElementTree as ET
from sqlalchemy import create_engine, update
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy import Table, Column, Integer, String, MetaData
log = getLogger('su.artix.queue')


class CardLabHandler(BaseHandler):
    """
    Обработчик сервиса очередей  для лояльности CardLab
    """

    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(CardLabHandler, self).__init__(dao, stopThreadsFlag, lastFailsLock, lastFails)
        self.messageType = "cardlab"
        self.name = "CardLabHandler"

    def sendMessage(self, message):
        log.info(u'Обработка сообщения %s, %s' % (message.messageid, message.body))
        try:
            body = json.loads(message.body)
            result, answer = self.send(message)
            if result['status'] == '200':
                answer = ET.fromstring(answer)
                result = answer.find(u'{transactionResponse}result')
                if result == None:
                    result = answer.find(u'{errorResponse}result')
                if result.text == '1':
                    log.info(u'Метод выполнен!')
                    xml = ET.fromstring(body[u'body'].encode('utf-8'))
                    action = xml.find(u'action')
                    print action.text
                    if action.text == 'buyNew':
                        cardId = body[u'cardid']
                        id = answer.find(u'{transactionResponse}transactionId')
                        self.updateCard(id.text, cardId)
                    self.deleteSentMessage(message)
                    log.info(u'Обработчик CardLabHandler. Сообщение %s успешно отправлено' % message.messageid)
                    return True
                else:
                    resultComment = answer.find(u'{errorResponse}resultComment')
                    log.error(u'Ошибка выполнения метода, код ошибки: %s, ошибка: %s' 
                                 % (result.text, resultComment.text))
            else:
                log.info(u'Обработчик CardLabHandler. Ошибка отправки сообщения %s' %message.messageid)
        except Exception as e:
            log.error(u'Ошибка: %s' % e)

        return False

    def updateCard(self, transactionId, cardId):
        log.info(u'Запись transactionId=%s для карты id=%s' %(transactionId, cardId))
        card = Table("carditem", MetaData(),
                      Column('carditemid', Integer, primary_key=True),
                      Column('document', String(200)))
        
        engine = create_engine('mysql://root@localhost/documents?charset=utf8', echo=False)
        conn = engine.connect()
        q = update(card).where(card.c.carditemid == cardId).values(document = transactionId)
        conn.execute(q)
        
    def send(self, message):
        content = json.loads(message.body)
        url = str(content[u'url'])
        xml = content[u'body']
        log.info(u'Отправка на адрес %s сообщения: %s' % (url, xml))
        conn = httplib2.Http(timeout = 10, disable_ssl_certificate_validation=True)
        result, answer =  conn.request(url, "POST", body=xml.encode('utf-8'))
        log.info(u'Ответ от сервера: %s , %s' % (result, answer.decode('utf-8')))
        
        return result, answer
