# -*- coding: utf8 -*-

from logging import getLogger
from base import BaseHandler
import socket
import json

log = getLogger('su.artix.queue')


class EpsHandler(BaseHandler):
    """
    Обработчик собщений для EPS Loyalty
    отправляет сообщение через TCP сокет и ждет подтверждения обработки от сервера
    """

    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(EpsHandler, self).__init__(dao, stopThreadsFlag, lastFailsLock, lastFails)
        self.messageType = "eps"
        self.name = "EpsHandler"
        self.socket = socket.socket()
        self.socket.settimeout(5)

    def sendMessage(self, message):
        log.info(u'Обработка сообщения %s, %s' % (message.messageid, message.body))

        if self.send(message):
            log.info(u'Обработчик Eps. Сообщение %s успешно отправлено' % message.messageid)
            self.deleteSentMessage(message)
            self.socket.close()
            return True

        log.info(u'Обработчик Eps. Ошибка отправки сообщения %s' % message.messageid)
        self.socket.close()
        return False

    def send(self, message):
        content = json.loads(message.body)
        host = content['host']
        port = content['port']
        messageData = str(content['message']).decode('hex')

        try:
            self.socket.connect((host, port))
            self.socket.sendall(messageData)
        except socket.timeout:
            log.warn(u'Не удалось отправить сообщение %s, истекло время ожидания ответа от сервера' % message.messageid)
            return False
        except Exception as e:
            log.warn(u'Не удалось отправить сообщение %s, ошибка: %s' % (message.messageid, e.message))
            return False

        awaitingAnswerId = messageData[3:11]

        while True:
            try:
                data = self.socket.recv(1024)

                if len(data) > 11 and data[3:11] == awaitingAnswerId:
                    return True
                else:
                    return False

            except socket.timeout:
                log.warn(u'Не удалось отправить сообщение %s, истекло время ожидания ответа от сервера' % message.messageid)
                break
            except Exception as e:
                log.warn(u'Не удалось получить ответ от сервера на сообщение %s, ошибка: %s' % (message.messageid, e.message))
                break

        return False
