# -*- coding: utf8 -*-

from logging import getLogger
from base import BaseHandler
import json
import os
import xml.etree.ElementTree as ET
import contextlib
import ssl
import tempfile

log = getLogger('su.artix.queue')


class GoldCrownHandler(BaseHandler):
    """
    Обработчик сервиса очередей  для лояльности GoldCrown
    """

    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(GoldCrownHandler, self).__init__(dao, stopThreadsFlag, lastFailsLock, lastFails)
        self.messageType = "goldcrown"
        self.name = "GoldCrownHandler"
        self.skipErrorMessage = False

    @contextlib.contextmanager
    def pfx_to_pem(self, pfx_path, pfx_password):
        
        openssl = __import__('OpenSSL')
        
        with tempfile.NamedTemporaryFile(suffix='.pem') as t_pem:
            f_pem = open(t_pem.name, 'wb')
            pfx = open(pfx_path, 'rb').read()
            p12 = openssl.crypto.load_pkcs12(pfx, pfx_password)
            f_pem.write(openssl.crypto.dump_privatekey(openssl.crypto.FILETYPE_PEM, p12.get_privatekey()))
            f_pem.write(openssl.crypto.dump_certificate(openssl.crypto.FILETYPE_PEM, p12.get_certificate()))
            ca = p12.get_ca_certificates()
            if ca is not None:
                for cert in ca:
                    f_pem.write(openssl.crypto.dump_certificate(openssl.crypto.FILETYPE_PEM, cert))
            f_pem.close()
            yield t_pem.name
            
    def sendMessage(self, message):
        log.info(u'Обработка сообщения %s, %s' % (message.messageid, message.body))
        try:
            result = self.send(message)
            if result.status_code == 200:
                answer = ET.fromstring(result.text)
                for status in answer.iter('status'):
                    code = status.find('code')
                    if code.text == '0':
                        log.info(u'Метод выполнен!')
                        self.deleteSentMessage(message)
                        log.info(u'Обработчик GoldCrownHandler. Сообщение %s успешно отправлено' % message.messageid)
                        return True
                    else:
                        description = status.find('description')
                        log.error(u'Ошибка выполнения метода, код ошибки: %s, ошибка: %s' 
                                     % (code.text, description.text))
            else:
                log.info(u'Обработчик GoldCrownHandler. Ошибка отправки сообщения %s' %message.messageid)
        except Exception as e:
            log.error(u'Ошибка: %s' % e)

        return False

    def send(self, message):
        requests = __import__('requests')
        
        content = json.loads(message.body)
        url = str(content[u'url'])
        xml = content[u'body']
        certificatePath = content[u'certificatePath']
        certificatePassword = content[u'certificatePassword']
        headers = content[u'headers']
        log.info(u'Отправка на адрес %s сообщения: %s' % (url, xml))
        
        with self.pfx_to_pem(certificatePath, certificatePassword) as cert:
            result = requests.post(url, cert=cert, data=xml, headers = headers)
        log.info(u'Ответ от сервера: %s , %s' % (result.status_code, result.text))
        
        return result
