# -*- coding: utf8 -*-

import json
import httplib2
import xml.etree.ElementTree as ET
from sqlalchemy import create_engine, update
from sqlalchemy import Table, Column, Integer, String, MetaData
from logging import getLogger
from base import BaseHandler
log = getLogger('su.artix.queue')

class LoymaxHandler(BaseHandler):
    """
    Обработчик сервиса очередей для лояльности Loymax
    """

    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(LoymaxHandler, self).__init__(dao, stopThreadsFlag, lastFailsLock, lastFails)
        self.messageType = "loymax"
        self.name = "LoymaxHandler"

    def sendMessage(self, message):
        log.info(u'Обработка сообщения %s, %s' % (message.messageid, message.body))
        try:
            body = json.loads(message.body)
            result, answer = self.send(body)
            if result['status'] == '200':
                answer = ET.fromstring(answer)
                updateCard = False
                result = answer.find(u'CancelPurchases/CancelPurchaseResponse')
                if result == None:
                    result = answer.find(u'ConfirmPurchases/ConfirmPurchaseResponse')
                if result == None:
                    result = answer.find(u'Discounts/DiscountResponse')
                    if result != None:
                        updateCard = True
                if result == None:
                    result = answer.find(u'Refunds/RefundResponse')

                if result.get('ErrorCode') == '0':
                    log.info(u'Метод выполнен!')
                    if updateCard:
                        cardId = body[u'cardId']
                        id = result.get('TransactionID')
                        self.updateCard(id, cardId)
                    self.deleteSentMessage(message)
                    log.info(u'Обработчик LoymaxHandler. Сообщение %s успешно отправлено' % message.messageid)
                    return True
                else:
                    log.error(u'Ошибка выполнения метода, код ошибки: %s, ошибка: %s' 
                                 % (result.get('ErrorCode'), result.get('ErrorMessage')))
            else:
                log.info(u'Обработчик LoymaxHandler. Ошибка отправки сообщения %s' %message.messageid)
        except Exception as e:
            log.error(u'Ошибка: %s' % e)

        return False

    def updateCard(self, transactionId, cardId):
        log.info(u'Запись transactionId=%s для карты id=%s' %(transactionId, cardId))
        card = Table("carditem", MetaData(),
                      Column('carditemid', Integer, primary_key=True),
                      Column('document', String(200)))

        engine = create_engine('mysql://root@localhost/documents?charset=utf8', echo=False)
        conn = engine.connect()
        q = update(card).where(card.c.carditemid==cardId).values(document=transactionId)
        conn.execute(q)
        conn.close()
        engine.dispose()
  
    def send(self, content):
        url = str(content[u'url'])
        headers = content[u'headers']
        headers = dict((k.encode('ascii') if isinstance(k, unicode) else k,
                        v.encode('ascii') if isinstance(v, unicode) else v)
                        for k,v in headers.items())
        xml = content[u'body']
        log.info(u'Отправка на адрес %s сообщения: %s' % (url, xml))
        conn = httplib2.Http(timeout=10, disable_ssl_certificate_validation=True)
        result, answer = conn.request(url, "POST", headers=headers, body=xml.encode('utf-8'))
        log.info(u'Ответ от сервера: %s , %s' % (result, answer.decode('utf-8')))

        return result, answer
