# -*- coding: utf8 -*-

import json
import httplib2
import xml.etree.ElementTree as ET
from logging import getLogger
from base import BaseHandler
log = getLogger('su.artix.queue')

class ManzanaHandler(BaseHandler):
    """
    Обработчик сервиса очередей для лояльности Loymax
    """

    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(ManzanaHandler, self).__init__(dao, stopThreadsFlag, lastFailsLock, lastFails)
        self.messageType = "manzana"
        self.name = "ManzanaHandler"

    def sendMessage(self, message):
        log.info(u'Обработка сообщения %s, %s' % (message.messageid, message.body))
        try:
            body = json.loads(message.body)
            result, answer = self.send(body)
            if result['status'] == '200':
                answer = ET.fromstring(answer)
                namespaces = {
                    'soap': 'http://www.w3.org/2003/05/soap-envelope',
                    'a': 'http://loyalty.manzanagroup.ru/loyalty.xsd',
                }
                chequeResponse = answer.find(u'soap:Body/a:ProcessRequestResponse/a:ProcessRequestResult/a:ChequeResponse', namespaces)
                returnCode = int(chequeResponse.find(u'a:ReturnCode', namespaces).text)
                errorText = chequeResponse.find(u'a:Message', namespaces).text

                if returnCode == 0:
                    self.deleteSentMessage(message)
                    log.info(u'Обработчик ManzanaHandler. Сообщение %s успешно отправлено' % message.messageid)
                    return True
                elif returnCode == 81301:
                    self.deleteSentMessage(message)
                    log.info(u'Обработчик ManzanaHandler. Сообщение %s уже было доставлено ранее' % message.messageid)
                    return True
                else:
                    log.error(u'Ошибка выполнения метода, код ошибки: %s, ошибка: %s' 
                                 % (returnCode, errorText))
            else:
                log.info(u'Обработчик ManzanaHandler. Ошибка отправки сообщения %s' %message.messageid)
        except Exception as e:
            log.error(u'Ошибка: %s' % e)

        return False
  
    @staticmethod
    def encode(str):
        return str.encode('ascii') if isinstance(str, unicode) else str

    def send(self, content):
        url = str(content[u'url'])
        headers = content[u'headers']
        headers = dict((self.encode(k), self.encode(v)) for k,v in headers.items())
        xml = content[u'body']
        log.info(u'Отправка на адрес %s сообщения: %s' % (url, xml))
        conn = httplib2.Http(timeout=15, disable_ssl_certificate_validation=True)
        result, answer = conn.request(url, "POST", headers=headers, body=xml.encode('utf-8'))
        log.info(u'Ответ от сервера: %s , %s' % (result, answer.decode('utf-8')))

        return result, answer
