# -*- coding: utf8 -*-

from logging import getLogger
from base import BaseHandler
import httplib2
import json
import xml.etree.ElementTree as ET

log = getLogger('su.artix.queue')

class RSLoyaltyHandler(BaseHandler):
    """
    Обработчик сервиса очередей для лояльности RSLoyalty
    """

    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(RSLoyaltyHandler, self).__init__(dao, stopThreadsFlag, lastFailsLock, lastFails)
        self.messageType = "rsloyalty"
        self.name = "RSLoyaltyHandler"

    def sendMessage(self, message):
        log.info(u'Обработка сообщения %s, %s' % (message.messageid, message.body))
        try:
            result, answer = self.send(message)
            if result['status'] == '200':
                answer = ET.fromstring(answer)
                element = answer.find('.//{http://tempuri.org/}CancelActivationPaymentCardResult')
                if element.text == u'true':
                    log.info(u'Метод выполнен!')
                    self.deleteSentMessage(message)
                    log.info(u'Обработчик RSLoyaltyHandler. Сообщение %s успешно отправлено' % message.messageid)
                    return True
                log.error(u'Обработчик RSLoyaltyHandler. Не удалось интерпретировать ответ для сообщения %s' % message.messageid)
            elif result['status'] == '500':
                answer = ET.fromstring(answer)
                element = answer.find('.//faultstring')
                if (element.text == u'Дисконтная карта не активна' or
                    element.text == u'Дисконтная карта не найдена'):
                    log.info(u'Ответ от RSLoyalty %s. Сообщение %s будет удалено' % (element.text, message.messageid))
                    self.deleteSentMessage(message)
                    return True
                log.error(u'Обработчик RSLoyaltyHandler. Необрабатываемая ошибка для сообщения %s' % message.messageid)
            else:
                log.error(u'Обработчик RSLoyaltyHandler. Ошибка отправки сообщения %s' % message.messageid)
        except Exception as e:
            log.error(u'Ошибка: %s' % e)

        return False

    def send(self, message):
        content = json.loads(message.body)
        url = str(content[u'url'])
        headers = content[u'headers']
        xml = content[u'body']
        log.info(u'Отправка на адрес %s сообщения: %s' % (url, xml))
        conn = httplib2.Http(timeout = 10, disable_ssl_certificate_validation = True)
        result, answer = conn.request(url, "POST", headers = headers, body = xml.encode('utf-8'))
        log.info(u'Ответ от сервера: %s , %s' % (result, answer.decode('utf-8')))

        return result, answer
