# -*- coding: utf8 -*-
from logging import getLogger
from base import BaseHandler
import httplib2
import json

log = getLogger('su.artix.queue')

class WaybillNumberHandler(BaseHandler):
    """
    Обработчик сообщений для обновления статуса номера универсального передаточного документа
    """
    def __init__(self, dao, stopThreadsFlag, lastFailsLock, lastFails):
        super(WaybillNumberHandler, self).__init__(dao, stopThreadsFlag, lastFailsLock, lastFails)
        self.messageType = "waybillnumber"
        self.name = "WaybillNumberHandler"
        self.conn = httplib2.Http(timeout = 10, disable_ssl_certificate_validation = True)

    def sendMessage(self, message):
        log.info(u'Обработка сообщения %s, %s' % (message.messageid, message.body))
        try:
            result, content = self.send(message)
            if result['status'] == '200':
                self.deleteSentMessage(message)
                log.info(u'Обработчик WaybillNumber. Сообщение %s успешно отправлено' % message.messageid)
                return True
            else:
                log.error(u'Ошибка REST-запроса, код ошибки: %s' % result['status'])
        except Exception as e:
            log.error(u'Ошибка: %s' % e)
        return False

    def send(self, message):
        content = json.loads(message.body)
        url = str(content[u'url'])
        messageBody = str(content[u'body'])
        log.info(u'Отправка на адрес %s сообщения: %s' % (url, messageBody))
        result, answer = self.conn.request(url, method = "PUT", headers = {'Content-Type': 'application/json; charset=UTF-8'}, body = messageBody)
        log.info(u'Ответ от сервера: %s , %s' % (result, answer.decode('utf-8')))
        return result, answer
